% Plot the error of a numerical method
% versus step size and computational time
% for the HME with saturable gain.
%clear
%close all

figure_filename1 = 'hme_error_vs_h';
figure_filename2 = 'hme_time_vs_h';

% With known parameters:
A = 0.075860120776239043616; tao = 43.633704550524000329; beta = 5.1102393784491929585;
phi = 0.022466811100191768191;
g0 = 3; Esat = 1; l = 2-0j*phi; Omega = sqrt(2); del = 1; gam = 4; b2 = 3; T = 1000;

sigma = 0;

% Numeric parameters
N = 1024;                            % T sampling points
zmax = 300;

hs = logspace(-2,2,20);

methods = {@ssfm_s2,@ssfm2,@ssfm4,@ark,@etdrk};
method_labels = {'SSFM(s)2', 'SSFM2', 'SSFM4', 'ARK', 'ETDRK'};
styles = {'gs--','kx-', 'bo--', 'r-.','m^-'};

% Setup
ss = ark();
ss.setup(N,T,Omega,g0,Esat,l,b2,del+1j*gam,sigma);
ss.doplot = false;

% Initial condition
u0 = 0.25*exp(-(ss.t/5).^2);
% u0 = A*sech(ss.t/tao).^(1+1j*beta);

% Accurate reference solution
href = min(hs)/4;
uref = ss.solve(u0,zmax,href);

% Calculate for different solvers
for ii=(1:length(methods))
    fprintf('Solving: %s\n',method_labels{ii})
    solver = methods{ii};
    
    % Create ETD solver
    ss = solver();
    ss.setup(N,T,Omega,g0,Esat,l,b2,del+1j*gam,sigma);
    ss.doplot = false;    
    
    % Calculate solution for different h
    runtimes = [];
    errors = [];
    for jj = 1:length(hs)
        h = hs(jj);
        fprintf('Solving for h=%.5g\n',h);
        
        u = ss.solve(u0,zmax,h);
        rerror = norm((u - uref))/norm(uref);
        
        if ss.zsolved<zmax || rerror>10
            break
        end
        
        errors = [errors rerror];
        runtimes = [runtimes ss.time_taken];
    end
    
    figure(1)
    loglog(hs(1:length(errors)),errors,styles{ii});
    grid on
    hold on
    
    figure(2)
    loglog(runtimes,errors,styles{ii});
    grid on
    hold on
    
end


% Set fonts and print figures
figure(1)
leg=legend(method_labels,'location','southeast');
xl=xlabel('Step size, \ith');
yl=ylabel('Relative error');
set(gca,'FontName','Times New Roman','FontSize',14);
set(xl,'FontSize',14,'FontName','Times New Roman');
set(yl,'FontSize',14,'FontName','Times New Roman');
set(leg,'FontSize',11,'FontName','Times New Roman');

set(gcf, 'PaperUnits', 'inches', 'PaperPositionMode', 'manual', 'PaperPosition', [0 0 4 3]*1.2);
set(gca,'XMinorGrid','off')

print(gcf,strcat(figure_filename1,'.eps'),'-depsc2');
system(sprintf('epstopdf %s --outfile=%s',strcat(figure_filename1,'.eps'),strcat(figure_filename1,'.pdf')));

figure(2)
legend(method_labels,'location','SouthWest');
xl=xlabel('Computational time, s');
yl=ylabel('Relative error');
set(gca,'FontName','Times New Roman','FontSize',14);
set(xl,'FontSize',14,'FontName','Times New Roman');
set(yl,'FontSize',14,'FontName','Times New Roman');
set(leg,'FontSize',11,'FontName','Times New Roman');

set(gcf, 'PaperUnits', 'inches', 'PaperPositionMode', 'manual', 'PaperPosition', [0 0 4 3]*1.2);
set(gca,'XMinorGrid','off')

print(gcf,strcat(figure_filename2,'.eps'),'-depsc2');
system(sprintf('epstopdf %s --outfile=%s',strcat(figure_filename2,'.eps'),strcat(figure_filename2,'.pdf')));

