
% ARK3(2)4 Method of Kennedy & Carpenter
% Here we simplify the method assuming that f_s is a linear function
% given by a diagonal matrix so
%
%  f_s(x) = Δ x
%

classdef ark <  hmevolution
    % numerical constants
    properties
        s = 6   % Number of stages

        gamma_ark = 1/4

        c = [0 1/2 83/250 31/50 17/20 1]

        % ERK coefficients
        ae = [
        1/2 0 0 0 0 0;
        13861/62500 6889/62500 0 0 0 0;
        -116923316275/2393684061468 -2731218467317/15368042101831 9408046702089/11113171139209 0 0 0;
        -451086348788/2902428689909 -2682348792572/7519795681897 12662868775082/11960479115383 3355817975965/11060851509271 0 0;
        647845179188/3216320057751 73281519250/8382639484533 552539513391/3454668386233 3354512671639/8306763924573 4040/17871 0
        ]

        ai = [
        1/4 1/4 0 0 0 0;
        8611/62500 -1743/31250 1/4 0 0 0;
        5012029/34652500 -654441/2922500 174375/388108 1/4 0 0;
        15267082809/155376265600 -71443401/120774400 730878875/902184768 2285395/8070912 1/4 0;
        82889/524892 0 15625/83664 69875/102672 -2260/8211 1/4
        ]

        b = [82889/524892 0 15625/83664 69875/102672 -2260/8211 1/4]
        bem = [4586570599/29645900160 0 178811875/945068544 814220225/1159782912 -3700637/11593932 61727/225920]
    end

    methods
        % First nonlinear piece
        function n1 = N1(self,v,fba)
            u = ifft(v);
            absu2 = abs(u).^2;
            n1 = self.deltagamma*absu2.*u - self.sigma*absu2.^2.*u - fba.*u;
            n1 = fft(n1);
        end

        % Second nonlinear piece
        function n2 = N2(self,v,g1)
            g = self.g0/(1 + norm(v)^2/self.Esatw);
            n2 = (g-g1)*self.L_gainsat.*v;
        end

        function [v,hstiff,rerror] = solver_singlestep(self,v,h,fba)
            hstiff = h;
            w = self.w;
            v0 = v;
            g1 = self.g0/(1 + norm(v0)^2/self.Esatw);
            g2 = -2*(g1^2/self.g0/self.Esatw);

            L = (self.L_dispersion + g1*self.L_gainsat);
            
            % Stiff implicit matrix
            Fsi = 1 - h*self.gamma_ark*L;

            % Stage values
            Fustage1(:,1) = self.N1(v,fba);
            Fustage2(:,1) = self.N2(v,g1);
            Austage(:,1) = L.*v;
            for ii=2:self.s
                sii = v;
                for jj=1:ii-1
                    afs_jj = self.ai(ii-1,jj)*Austage(:,jj);
                    afns_jj1 = self.ae(ii-1,jj)*Fustage1(:,jj);
                    afns_jj2 = self.ae(ii-1,jj)*Fustage2(:,jj);

                    sii = sii + h*(afs_jj + afns_jj1 + afns_jj2);
                end
                
                % Solve implicitly for each stage
                ustage = sii./Fsi;

                Austage(:,ii) = L.*ustage;
                Fustage1(:,ii) = self.N1(ustage,fba);
                Fustage2(:,ii) = self.N2(ustage,g1);
            end
            
            % Final stage
            vapprox = v;
            for jj=1:self.s
                FAustage = (Austage(:,jj) + Fustage1(:,jj) + Fustage2(:,jj));

                v = v + h*self.b(jj)*FAustage;
                vapprox = vapprox + h*self.bem(jj)*FAustage;
            end            

            aerror = norm(v-vapprox);
            rerror = norm(v-vapprox)/norm(v);
        end % solve_step

        
    end %Methods
end %imexark3

