% Class to make spectral transformations easier
classdef etdrk < hmevolution

    methods
        function [E,E2,Q,f1,f2,f3]=etd_coefficients(self,h,Lh)
            % Switch to different methods to avoid rounding errors
            [E,E2,Q,f1,f2,f3]=self.etd_coefficients_direct(h,Lh);
            
            ssel = abs(Lh)<0.1;
            [Es,E2s,Qs,f1s,f2s,f3s]=self.etd_coefficients_series(h,Lh(ssel));

            E(ssel) = Es;
            E2(ssel) = E2s;
            Q(ssel) = Qs;
            f1(ssel) = f1s;
            f2(ssel) = f2s;
            f3(ssel) = f3s;
        end

        function [E,E2,Q,f1,f2,f3]=etd_coefficients_direct(self,h,Lh)
            % Calculate ETD coefficients
            E = exp(Lh);
            E2 = exp(Lh/2);

            % Direct calulation
            Q = h*expm1(Lh/2)./Lh;
            f1 = h*(-4-Lh+E.*(4-3*Lh+Lh.^2))./Lh.^3;
            f2 = h*(2+Lh+E.*(-2+Lh))./Lh.^3;
            f3 = h*(-4-3*Lh-Lh.^2+E.*(4-Lh))./Lh.^3;
        end

        function [E,E2,Q,f1,f2,f3]=etd_coefficients_series(self,h,Lh)
            % Calculate ETD coefficients
            E = exp(Lh);
            E2 = exp(Lh/2);

            % Taylor approximants
            Lh2 = Lh.^2;
            Lh3 = Lh.^3;
            Q = h*(1/2 + Lh/8 + Lh2/48 + Lh3/384);
            f1 = h*(1/6 + Lh/6 + 3*Lh2/40 + Lh3/45);
            f2 = h*(1/6 + Lh/12 + Lh2/40 + Lh3/180);
            f3 = h*(1/6 - Lh2/120 - Lh3/360);
        end

        function [E,E2,Q,f1,f2,f3]=etd_coefficients_pade22(self,h,Lh)
            % Calculate ETD coefficients
            E = exp(Lh);
            E2 = exp(Lh/2);

            % Taylor approximants
            Lh2 = Lh.^2;
            Q = h*(1/2+(Lh)/40+(Lh2)/480)./(1-Lh/5+Lh2/80);
            f1 = h*(1/6+(109*Lh)/1162+(2657*Lh2)/209160)./(1-(254*Lh)/581+(221*Lh2)/3486);
            f2 = h*(1/6+(5*Lh)/196+(41*Lh2)/17640)./(1-(17*Lh)/49+(11*Lh2)/294);
            f3 = h*(1/6-(Lh)/18-(13*Lh2)/7560)./(1-Lh/3 + (5*Lh2)/126);
        end


        % First nonlinear piece
        function n1 = N1(self,v,fba)
            u = ifft(v);
            absu2 = abs(u).^2;
            n1 = self.deltagamma*absu2.*u - self.sigma*absu2.^2.*u - fba.*u;
            n1 = fft(n1);
        end

        % Second nonlinear piece
        function n2 = N2(self,v,g1)
            g = self.g0/(1 + norm(v)^2/self.Esatw);
            n2 = (g-g1)*self.L_gainsat.*v;
        end

        function [v,hstiff,rerror] = solver_singlestep(self,v,h,fba)
            rerror = -1;
            hstiff = h;
            w = self.w;
            
            %Linear operator
            g1 = self.g0/(1 + norm(v)^2/self.Esatw);
            Lh = h*(self.L_dispersion + g1*self.L_gainsat);

            % Calculate coefficients
            [E,E2,Q,f1,f2,f3]=self.etd_coefficients_pade22(h,Lh);

            % ETD stages
            Nv = self.N1(v,fba) + self.N2(v,g1);
            a = E2.*v + Q.*Nv;
            Na = self.N1(a,fba) + self.N2(a,g1);
            b = E2.*v + Q.*Na;
            Nb = self.N1(b,fba) + self.N2(b,g1);
            c = E2.*a + Q.*(2*Nb-Nv);
            Nc = self.N1(c,fba) + self.N2(c,g1);
            v = E.*v + Nv.*f1 + 2*(Na+Nb).*f2 + Nc.*f3;
        end

    end % Methods

end % Classdef
