% Class to make constructing solvers easier
%
% The class is to collect common routines for solvin the HME
%
% 
%
%
classdef hmevolution < handle
        
    % Algorithm properties
    properties
        doplot = 1
        rtol = 0
        method_variant = 0
        stability_control = false
        error_control = false
        method_information = 0
    end

    properties
        Esatw
        Omega
        g0
        Esat
        l
        b2
        deltagamma
        sigma
    end

    % Internal variables
    properties (GetAccess='public',SetAccess='public')
        N
        T
        dt
        dw
        naxis
        t
        w
        L_gainsat
        L_dispersion

        L_gainsat_a
        L_dispersion_a
        L_dispersion_b
    end
            
    % Outputs
    properties (GetAccess='public',SetAccess='private')
        tplot
        zplot
        uplot
        Eplot
        time_taken

        nsteps
        zsteps
        zsolved
        
        rerrors
        rerror_max
    end

    methods
        function obj=hmevolution()
        end

        function setup(self,N,T,Omega,g0,Esat,l,b2,deltagamma,sigma)
            self.N = N;
            self.T = T;
            self.naxis = [-N/2:1:N/2-1]';

            self.dt = self.T/self.N;                   % temporal resolution
            self.t = self.naxis*self.dt;                         % time axis

            self.dw = 2*pi/self.T;                     % Freq resolution
            self.w = fftshift(self.naxis*self.dw);                         % Freq axis

            % Equation parameters
            self.g0=g0;
            self.Esat=Esat;
            self.l=l;
            self.b2=b2;
            self.deltagamma=deltagamma;
            self.sigma=sigma;
            self.Omega = Omega;

            % Saturated energy in Fourier domain
            self.Esatw = self.Esat*self.N/self.dt;
            
            % Precalculate vectors to speed things up
            self.L_gainsat = (1-self.w.^2/self.Omega^2)/2;
            self.L_dispersion = (1i*self.b2*self.w.^2 - self.l)/2;

            % For real/imaginary methods
            self.L_dispersion_a = -real(l)/2;
            self.L_dispersion_b = -imag(l)/2 + self.b2/2*self.w.^2;
            self.L_gainsat_a = (1-self.w.^2/self.Omega^2)/2;
        end

        function u=solve(self,u0,zmax,h,varargin)
            optargin = size(varargin,2);
            if optargin>0
                if strcmp(varargin{1},'rtol')
                    self.rtol = varargin{2};
                end
            end

            % Intial pulse in Fourier domain
            v = fft(u0);

            % Boundary absorber
            fba = 0;
            
            % Save the pulse shape each dzplot
            if self.doplot
                dzplot = zmax/100;
                zplot_current = 0;

                self.tplot = self.t(1:8:end);
                self.zplot = [];
                self.uplot = [];
                self.Eplot = [];
            end
            
            self.rerrors = [];
            self.zsteps = [];
            self.rerror_max = 0;
            hz_changed = false;
            z = 0;
            nsteps = 0;
            tic;
            while (z+h/2)<zmax
                [vnext,hstiff,rerror] = self.solver_singlestep(v,h,fba);
                
                hz_changed = false;
                accept_current_z = true;
                
                % Stiffness detection
                if self.stability_control && h>hstiff
                    h_next = hstiff/2;
                    accept_current_z = false;
                    hz_changed = true;

                % I controller for step-size, p=3, q=4
                elseif self.error_control
                    h_next = 0.8*h*(self.rtol/rerror)^(1/4);
                    hz_changed = true;
                end

                if any(isnan(vnext))
                    fprintf('Error: NaN in solution z=%g\n', z);
                    break;
                end

                % Update solution
                if accept_current_z
                    z = z+h;
                    v = vnext;

                    self.zsteps(end+1) = z;
                    self.rerrors(end+1) = rerror;
                    self.rerror_max = max(self.rerror_max,rerror);

                    if self.doplot & z>=zplot_current
                        self.saveplot(v, z);
                        zplot_current = zplot_current + dzplot;
                    end
                end

                % Change stepsize after update
                if hz_changed
                    h = h_next;
                    %fprintf('Stepsize change: %.2g, error: %.3g\n', h, rerror);
                end

                % Make sure last step is of correct size
                if z+h>zmax
                    h = zmax-z;
                end
                nsteps = nsteps + 1;
            end
            self.nsteps = nsteps;       % Number of z-steps to solve
            self.zsolved = z;           % Save z position
            self.time_taken = toc;

            % Output pulse
            u = ifft(v);
        end

        function saveplot(self, v, z)
            fprintf('.');
            self.zplot(end+1) = z;
            utemp = ifft(v);
            self.uplot(end+1,:) = abs(utemp(1:8:end));
            self.Eplot(end+1) = norm(v)^2*self.dt;
        end

        function plot_evolution(self)
            subplot(211)
            plot(self.zplot,self.Eplot,'k-');
            xlabel('z')
            ylabel('E')
            title('Evolution of pulse');

            subplot(212);
            [Zp,Tp]=meshgrid(self.zplot,self.tplot);
            surf(Zp,Tp,self.uplot');
            shading interp;
            axis tight;
            view([-90 90]);
            colormap(hot);
        end


    end % Methods

end % Classdef
